@@ -44,6 +44,7 @@ void trap (void);
 #include "iatt.h"
 #include "uuid.h"
 #include "libglusterfs-messages.h"
+#include "protocol-common.h"
 
 #define STRINGIFY(val) #val
 #define TOSTRING(val) STRINGIFY(val)
@@ -52,8 +53,8 @@ void trap (void);
 
 #define min(a,b) ((a)<(b)?(a):(b))
 #define max(a,b) ((a)>(b)?(a):(b))
-#define roof(a,b) ((((a)+(b)-1)/((b)?(b):1))*(b))
-#define floor(a,b) (((a)/((b)?(b):1))*(b))
+#define roof(a,b) ((((a)+(b)-1)/((b!=0)?(b):1))*(b))
+#define floor(a,b) (((a)/((b!=0)?(b):1))*(b))
 
 #define IPv4_ADDR_SIZE 32
 
@@ -101,7 +102,7 @@ void trap (void);
 #define GF_CLNT_INSECURE_PORT_CEILING (GF_IANA_PRIV_PORTS_START - 1)
 #define GF_PORT_MAX 65535
 #define GF_PORT_ARRAY_SIZE ((GF_PORT_MAX + 7) / 8)
-
+#define GF_LOCK_TIMER 180
 #define GF_MINUTE_IN_SECONDS 60
 #define GF_HOUR_IN_SECONDS (60*60)
 #define GF_DAY_IN_SECONDS (24*60*60)
@@ -113,6 +114,10 @@ void trap (void);
 /* Default value of signing waiting time to sign a file for bitrot */
 #define SIGNING_TIMEOUT "120"
 
+/* xxhash */
+#define GF_XXH64_DIGEST_LENGTH 8
+#define GF_XXHSUM64_DEFAULT_SEED 0
+
 /* Shard */
 #define GF_XATTR_SHARD_FILE_SIZE  "trusted.glusterfs.shard.file-size"
 #define SHARD_ROOT_GFID "be318638-e8a0-4c6d-977d-7a937aa84806"
@@ -126,11 +131,15 @@ void trap (void);
 
 #define GF_PERCENTAGE(val, total) (((val)*100)/(total))
 
-enum _gf_boolean
-{
-	_gf_false = 0,
-	_gf_true = 1
-};
+/* pthread related */
+#define GF_THREAD_NAMEMAX 9
+#define GF_THREAD_NAME_PREFIX "gluster"
+#define GF_THREAD_NAME_PREFIX_LEN 7
+
+#include <stdbool.h>
+#define gf_boolean_t    bool
+#define _gf_false       false
+#define _gf_true        true
 
 /*
  * we could have initialized these as +ve values and treated
@@ -160,7 +169,6 @@ enum _gf_xlator_ipc_targets {
         GF_IPC_TARGET_UPCALL = 2
 };
 
-typedef enum _gf_boolean gf_boolean_t;
 typedef enum _gf_special_pid gf_special_pid_t;
 typedef enum _gf_xlator_ipc_targets _gf_xlator_ipc_targets_t;
 
@@ -414,6 +422,14 @@ BIT_VALUE (unsigned char *array, unsigned int index)
         } while (0)
 #endif
 
+#define GF_ABORT(msg)                                                   \
+        do {                                                            \
+                gf_msg_callingfn ("", GF_LOG_CRITICAL, 0,               \
+                                  LG_MSG_ASSERTION_FAILED,              \
+                                  "Assertion failed: " msg);            \
+                abort();                                                \
+        } while (0)
+
 #define GF_UUID_ASSERT(u) \
         if (gf_uuid_is_null (u))\
                 GF_ASSERT (!"uuid null");
@@ -435,7 +451,7 @@ union gf_sock_union {
 
 #define IOV_MIN(n) min(IOV_MAX,n)
 
-#define GF_FOR_EACH_ENTRY_IN_DIR(entry, dir, scr) \
+#define GF_SKIP_IRRELEVANT_ENTRIES(entry, dir, scr) \
         do {\
                 entry = NULL;\
                 if (dir) { \
@@ -678,12 +694,15 @@ gf_time_fmt (char *dst, size_t sz_dst, time_t utime, unsigned int fmt)
         static gf_timefmts timefmt_last = (gf_timefmts) - 1;
         static const char **fmts;
         static const char **zeros;
-        struct tm tm;
+        struct tm tm, *res;
+        int localtime = 0;
 
         if (timefmt_last == (gf_timefmts) - 1)
                 _gf_timestuff (&timefmt_last, &fmts, &zeros);
         if (timefmt_last < fmt) fmt = gf_timefmt_default;
-        if (utime && gmtime_r (&utime, &tm) != NULL) {
+        localtime = gf_log_get_localtime ();
+        res = localtime ? localtime_r (&utime, &tm) : gmtime_r (&utime, &tm);
+        if (utime && res != NULL) {
                 strftime (dst, sz_dst, fmts[fmt], &tm);
         } else {
                 strncpy (dst, "N/A", sz_dst);
@@ -774,6 +793,14 @@ void skip_word (char **str);
 /* returns a new string with nth word of given string. n>=1 */
 char *get_nth_word (const char *str, int n);
 
+typedef struct token_iter {
+        char *end;
+        char sep;
+} token_iter_t;
+char *token_iter_init (char *str, char sep, token_iter_t *tit);
+gf_boolean_t next_token (char **tokenp, token_iter_t *tit);
+void drop_token (char *token, token_iter_t *tit);
+
 gf_boolean_t mask_match (const uint32_t a, const uint32_t b, const uint32_t m);
 gf_boolean_t gf_is_ip_in_net (const char *network, const char *ip_str);
 char valid_host_name (char *address, int length);
@@ -814,14 +841,18 @@ gf_ports_reserved (char *blocked_port, unsigned char *ports, uint32_t ceiling);
 int gf_get_hostname_from_ip (char *client_ip, char **hostname);
 gf_boolean_t gf_is_local_addr (char *hostname);
 gf_boolean_t gf_is_same_address (char *host1, char *host2);
-void md5_wrapper(const unsigned char *data, size_t len, char *md5);
+void gf_xxh64_wrapper(const unsigned char *data, size_t len,
+                      unsigned long long seed, char *xxh64);
 int gf_set_timestamp  (const char *src, const char* dest);
 
 int gf_thread_create (pthread_t *thread, const pthread_attr_t *attr,
-                      void *(*start_routine)(void *), void *arg);
+                      void *(*start_routine)(void *), void *arg,
+                      const char *name);
 int gf_thread_create_detached (pthread_t *thread,
-                      void *(*start_routine)(void *), void *arg);
-
+                      void *(*start_routine)(void *), void *arg,
+                      const char *name);
+gf_boolean_t
+gf_is_pid_running (int pid);
 gf_boolean_t
 gf_is_service_running (char *pidfile, int *pid);
 gf_boolean_t
@@ -899,7 +930,34 @@ gf_bits_index (uint64_t n);
 const char*
 gf_fop_string (glusterfs_fop_t fop);
 
+int
+gf_fop_int (char *fop);
+
 char *
 get_ip_from_addrinfo (struct addrinfo *addr, char **ip);
 
+int
+close_fds_except (int *fdv, size_t count);
+
+int
+gf_getgrouplist (const char *user, gid_t group, gid_t **groups);
+
+int
+glusterfs_compute_sha256 (const unsigned char *content, size_t size,
+                          char *sha256_hash);
+char*
+get_struct_variable (int mem_num, gf_gsync_status_t *sts_val);
+
+char *
+gf_strncpy (char *dest, const char *src, const size_t dest_size);
+
+void
+gf_strTrim (char **s);
+
+int
+gf_replace_old_iatt_in_dict (struct _dict *);
+
+int
+gf_replace_new_iatt_in_dict (struct _dict *);
+
 #endif /* _COMMON_UTILS_H */
