@@ -65,6 +65,8 @@ enum gf_fop_procnum {
         GFS3_OP_LEASE,
         GFS3_OP_GETACTIVELK,
         GFS3_OP_SETACTIVELK,
+        GFS3_OP_ICREATE,
+        GFS3_OP_NAMELINK,
         GFS3_OP_MAXVALUE,
 };
 
@@ -106,7 +108,7 @@ enum gf_pmap_port_type {
         GF_PMAP_PORT_FREE = 0,
         GF_PMAP_PORT_FOREIGN,     /* it actually means, not sure who is using it, but it is in-use */
         GF_PMAP_PORT_LEASED,
-        GF_PMAP_PORT_NONE,
+        GF_PMAP_PORT_ANY,
         GF_PMAP_PORT_BRICKSERVER, /* port used by brick process */
 };
 typedef enum gf_pmap_port_type gf_pmap_port_type_t;
@@ -134,6 +136,7 @@ enum gf_deprobe_resp {
         GF_DEPROBE_FRIEND_DOWN,
         GF_DEPROBE_QUORUM_NOT_MET,
         GF_DEPROBE_FRIEND_DETACHING,
+        GF_DEPROBE_SNAP_BRICK_EXIST,
 };
 
 enum gf_cbk_procnum {
@@ -147,6 +150,8 @@ enum gf_cbk_procnum {
         GF_CBK_CHILD_DOWN,
         GF_CBK_RECALL_LEASE,
         GF_CBK_STATEDUMP,
+        GF_CBK_INODELK_CONTENTION,
+        GF_CBK_ENTRYLK_CONTENTION,
         GF_CBK_MAXVALUE,
 };
 
@@ -200,6 +205,7 @@ enum gluster_cli_procnum {
         GLUSTER_CLI_GET_STATE,
         GLUSTER_CLI_RESET_BRICK,
         GLUSTER_CLI_REMOVE_TIER_BRICK,
+        GLUSTER_CLI_ADD_TIER_BRICK,
         GLUSTER_CLI_MAXVALUE,
 };
 
@@ -235,6 +241,7 @@ enum glusterd_brick_procnum {
         GLUSTERD_BRICK_BARRIER,
         GLUSTERD_NODE_BITROT,
         GLUSTERD_BRICK_ATTACH,
+        GLUSTERD_DUMP_METRICS,
         GLUSTERD_BRICK_MAXVALUE,
 };
 
@@ -263,6 +270,7 @@ typedef enum {
         GF_SHD_OP_SBRAIN_HEAL_FROM_LATEST_MTIME,
         GF_SHD_OP_GRANULAR_ENTRY_HEAL_ENABLE,
         GF_SHD_OP_GRANULAR_ENTRY_HEAL_DISABLE,
+        GF_SHD_OP_HEAL_SUMMARY,
 } gf_xl_afr_op_t ;
 
 struct gf_gsync_detailed_status_ {
@@ -329,6 +337,8 @@ typedef enum gf_get_snapshot_info_type gf_get_snapshot_info_type;
 #define GLUSTER_FOP_VERSION   330 /* 3.3.0 */
 #define GLUSTER_FOP_PROCCNT   GFS3_OP_MAXVALUE
 
+#define GLUSTER_FOP_VERSION_v2   400 /* 4.0.0 */
+
 /* Aggregator */
 #define GLUSTER_AGGREGATOR_PROGRAM 29852134 /* Completely random */
 #define GLUSTER_AGGREGATOR_VERSION 1
@@ -355,4 +365,5 @@ typedef enum gf_get_snapshot_info_type gf_get_snapshot_info_type;
 
 #define GD_VOLUME_NAME_MAX 256 /* Maximum size of volume name */
 
+#define GLUSTER_PROCESS_UUID_FMT "CTX_ID:%s-GRAPH_ID:%d-PID:%d-HOST:%s-PC_NAME:%s-RECON_NO:%s"
 #endif /* !_PROTOCOL_COMMON_H */
